
package org.wso2.solutions.identity.sts;

import java.security.Principal;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.apache.axis2.context.MessageContext;
import org.apache.rahas.RahasData;
import org.apache.rahas.TrustException;
import org.wso2.solutions.identity.sts.users.IPUser;
import org.wso2.usermanager.PrincipalNotFoundException;
import org.wso2.usermanager.realm.generic.GenericJNDIRealm;

/**
 * 
 * @author dimuthu leelarathne (muthulee AT apache.org)
 *
 */
public class IdentityProviderData {

	protected String cardID = null;
	protected Map requestedClaims = null;
	protected String displayTokenLang = null;

//	protected RahasData rahasData = null;
	/**
	 * 
	 * @param inMessageContext
	 * @param data
	 * @throws TrustException
	 */
	public IdentityProviderData(RahasData data) throws IdentityProviderException {


		OMElement rstElem = data.getRstElement();
		OMElement claimElem = data.getClaimElem();

		if(claimElem!=null){
			processClaimsData(data,claimElem);
		}
		processInfoCardReference(rstElem);
		
		populateClaimValues(data);

	}

	/**
	 * 
	 * @param rst  This is a RequestSecurityToken
	 * @param data
	 */
	private void processInfoCardReference(OMElement rst) throws IdentityProviderException{
		OMElement infoCardRef = rst.getFirstChildWithName(
				new QName(IdentityConstants.IDENTITY_NAMESPACE,IdentityConstants.LocalNames.INFO_CARD_REFERENCE));

		OMElement omCardID = infoCardRef.getFirstChildWithName(
				new QName(IdentityConstants.IDENTITY_NAMESPACE,IdentityConstants.LocalNames.CARD_ID));

		this.cardID = omCardID.getText();


	}

	/**
	 * 
	 * @param rst
	 * @param data
	 */
	private void processClaimsData(RahasData rahasData, OMElement claims) 
	throws IdentityProviderException{

		if(rahasData.getClaimDialect() != IdentityConstants.IDENTITY_NAMESPACE){
			throw new IdentityProviderException("Unknown dialect. Cannot accpet claims");
		}

		Iterator ite = claims.getChildrenWithName(new QName(IdentityConstants.IDENTITY_NAMESPACE, IdentityConstants.LocalNames.IDENTITY_CLAIM ));
		while(ite.hasNext()){
			OMElement omElem = (OMElement)ite.next();
			Claim claim = new Claim();
			String uriClaim = omElem.getAttributeValue(new QName(null, "URI")); 
			
			if(uriClaim == null){
				throw new IdentityProviderException("Claim URI cannot be empty");
			}
			claim.URI = uriClaim;
			String optional = (omElem.getAttributeValue(new QName(null, "Optional")));
			if(optional != null){
				claim.bOptional = (optional.equals("true"))?true:false;
			}else{
				claim.bOptional = true;
			}

			requestedClaims.put(claim.URI, claim);
		}

	}
	
	
	
	
	public String getCardID() {
		return cardID;
	}


	public String getValueForClaim(String claim){
		return null;
	}


	public String getDisplayTokenLang() {
		return displayTokenLang;
	}



	public Map getRequestedClaims() {
		return requestedClaims;
	}

	public class Claim{
		public String value;
		public String URI;
		public boolean bOptional;
	}
	
	protected String getDisplayName(String uriClaim){
		if(IdentityConstants.ATTR_NS_GIVEN_NAME.equals(uriClaim)){
			return IdentityConstants.DISPLAY_GIVEN_NAME;
		}else if(IdentityConstants.ATTR_NS_SURNAME.equals(uriClaim)){ 
			return IdentityConstants.DISPLAY_SURNAME;
		}else if(IdentityConstants.ATTR_NS_EMAIL_ADDRESS.equals(uriClaim)){
			return IdentityConstants.DISPLAY_EMAIL_ADDRESS;			
		}else if(IdentityConstants.ATTR_NS_STREET_ADDRESS.equals(uriClaim)){
			return IdentityConstants.DISPLAY_EMAIL_ADDRESS;
		}else if(IdentityConstants.ATTR_NS_CITY.equals(uriClaim)){ 
			return IdentityConstants.DISPLAY_CITY;
		}else if(IdentityConstants.ATTR_NS_STATE.equals(uriClaim)){
			return IdentityConstants.DISPLAY_STATE;
		}else if(IdentityConstants.ATTR_NS_POSTAL_CODE.equals(uriClaim)){
			return IdentityConstants.DISPLAY_POSTAL_CODE;
		}else if(IdentityConstants.ATTR_NS_COUNTRY.equals(uriClaim)){
			return IdentityConstants.DISPLAY_COUNTRY;	
		}else if(IdentityConstants.ATTR_NS_HOME_PHONE.equals(uriClaim)){
			return IdentityConstants.DISPLAY_HOME_PHONE;	
		}else if(IdentityConstants.ATTR_NS_WORK_PHONE.equals(uriClaim)){
			return IdentityConstants.DISPLAY_WORK_PHONE;	
		}else if(IdentityConstants.ATTR_NS_MOBILE_PHONE.equals(uriClaim)){
			return IdentityConstants.DISPLAY_MOBILE_PHONE;		
		}else{
			return null;
		}
	}
	
	protected void populateClaimValues(RahasData rahasData) throws IdentityProviderException{
		
		/* TODO
		 * The following is a hack for getting user things.
		 * When the MessageContext is given to the Passwordcallback then
		 * revert this.
		 */
		IPUser ipUser = (IPUser)getUser(rahasData);
		
		Iterator ite = requestedClaims.values().iterator();
		Claim claim = null;
		MessageContext msgCxt = rahasData.getInMessageContext();
				
		while(ite.hasNext()){
			claim = (Claim) ite.next();
			String uriClaim = claim.URI;
			String value = null;
			if(IdentityConstants.ATTR_NS_GIVEN_NAME.equals(uriClaim)){
				value = ipUser.getFirstName();
			}else if(IdentityConstants.ATTR_NS_SURNAME.equals(uriClaim)){ 
				value = ipUser.getLastName();
			}else if(IdentityConstants.ATTR_NS_EMAIL_ADDRESS.equals(uriClaim)){
				value = ipUser.getEmailAddress();			
			}else if(IdentityConstants.ATTR_NS_STREET_ADDRESS.equals(uriClaim)){
				value = ipUser.getStreetAddress();
			}else if(IdentityConstants.ATTR_NS_CITY.equals(uriClaim)){ 
				value = ipUser.getCity();
			}else if(IdentityConstants.ATTR_NS_STATE.equals(uriClaim)){
				value = ipUser.getState();
			}else if(IdentityConstants.ATTR_NS_POSTAL_CODE.equals(uriClaim)){
				value = ipUser.getPostalCode();
			}else if(IdentityConstants.ATTR_NS_COUNTRY.equals(uriClaim)){
				value = ipUser.getCountry();	
			}else if(IdentityConstants.ATTR_NS_HOME_PHONE.equals(uriClaim)){
				value = ipUser.getPrimaryPhoneNo();	
			}else if(IdentityConstants.ATTR_NS_WORK_PHONE.equals(uriClaim)){
				value = ipUser.getSecondaryPhoneNo();	
			}else if(IdentityConstants.ATTR_NS_MOBILE_PHONE.equals(uriClaim)){
				value = ipUser.getMobilePhoneNo();		
			}else{
				throw new IdentityProviderException();
			}
		}
	}
	
	
	/* TODO
	 * This method is a hack. Only call this service once.
	 * So remove this when there is another way.
	 */
	public IPUser getUser(RahasData rahasData){
		Principal principal = rahasData.getPrincipal();
		String username =  principal.getName();
		
		IPUser user = null;
    	//TODO:: put these things into a configuration file
    	GenericJNDIRealm realm = new GenericJNDIRealm();
		
		realm.setConnectionName("cn=root,dc=wso2,dc=com");
		realm.setConnectionPassword("secret");
		realm.setConnectionURL("ldap://10.100.1.138");
		
		realm.setDigestAlgo("MD5");
		realm.setPasswordAttrId("userPassword");

		Map map = new HashMap();

		map.put("password", "userPassword");
		map.put("name", "givenName");
		
		realm.setUserPattern("uid={0},ou=People,dc=wso2,dc=com");
		realm.configureUserBaseDriver(map, "org.wso2.solutions.identity.sts.users.IPUser");
		
		
		try {
			user = (IPUser) realm.getUser(username);
		} catch (PrincipalNotFoundException e) {
			e.printStackTrace();
		}
      	return user;
	}

}
