package org.wso2.solutions.identity.sts;

import java.security.Key;
import java.security.cert.X509Certificate;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMNode;
import org.apache.axiom.om.impl.dom.jaxp.DocumentBuilderFactoryImpl;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axis2.context.MessageContext;
import org.apache.rahas.RahasConstants;
import org.apache.rahas.RahasData;
import org.apache.rahas.Token;
import org.apache.rahas.TokenIssuer;
import org.apache.rahas.TrustException;
import org.apache.rahas.TrustUtil;
import org.apache.ws.security.WSConstants;
import org.apache.ws.security.WSSecurityException;
import org.apache.ws.security.components.crypto.Crypto;
import org.apache.ws.security.components.crypto.CryptoFactory;
import org.apache.ws.security.message.WSSecEncryptedKey;
import org.apache.ws.security.util.XmlSchemaDateFormat;
import org.apache.xml.security.signature.XMLSignature;
import org.opensaml.SAMLAssertion;
import org.opensaml.SAMLAttribute;
import org.opensaml.SAMLAttributeStatement;
import org.opensaml.SAMLException;
import org.opensaml.SAMLStatement;
import org.opensaml.SAMLSubject;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.wso2.solutions.identity.sts.IdentityProviderData.Claim;
import org.wso2.usermanager.PrincipalNotFoundException;
import org.wso2.usermanager.realm.generic.GenericJNDIRealm;

/**
 * 
 * 
 * @author dimuthu leearathne (dimuthul AT wso2.com)
 * @author ruchith fernando   (ruchith AT wso2.com)
 */
public class IdentityTokenIssuer implements TokenIssuer {

	    String confileFilePath;

	    public final static String ISSUER_SELF = IdentityConstants.NS_IDENTITY + "/issuer/self";


	    public String getResponseAction(RahasData data) throws TrustException {
	        return RahasConstants.WST_NS_05_02 + RahasConstants.RSTR_ACTION_ISSUE;
	    }

	    public SOAPEnvelope issue(RahasData data) throws TrustException {
	       try {
				IdentityProviderData stsData = new IdentityProviderData(data);
				return createResponse(data, stsData);
			} catch (IdentityProviderException e) {
				e.printStackTrace();
				throw new TrustException(e.getMessage());
			}
	    	
	    	
	     
	    }

	    /**
	     * @param data
	     * @return
	     * @throws TrustException
	     */
	    private SOAPEnvelope createResponse(RahasData rahasData, IdentityProviderData ipData) throws TrustException {
	        try {

	            Date notBefore = new Date();
	            Date notAfter = new Date();
	            notAfter.setTime(notBefore.getTime() + (300 * 1000));

	            // Set the DOM impl to DOOM
	            DocumentBuilderFactoryImpl.setDOOMRequired(true);
	            
	            MessageContext inMsgCtx = rahasData.getInMessageContext(); 
	            //.getInMessageContext();

	            SOAPEnvelope env = TrustUtil.createSOAPEnvelope(inMsgCtx
	                    .getEnvelope().getNamespace().getNamespaceURI());

	            Document doc = ((Element) env).getOwnerDocument();

	            Crypto crypto = CryptoFactory.getInstance("demoCrypto.properties");

	            // creating encrypted key- START
	            X509Certificate serviceCert = (X509Certificate) crypto
	                    .getKeyStore().getCertificate("bob");

	            // Create EncryptedKey
	            WSSecEncryptedKey encryptedKey = new WSSecEncryptedKey();

	            encryptedKey.setUseThisCert(serviceCert);
	            encryptedKey.setKeyIdentifierType(WSConstants.THUMBPRINT_IDENTIFIER);
	            encryptedKey.prepare(doc, null);

	            Element encrKeyElem = encryptedKey.getEncryptedKeyElement();

	            // creating encrypted key- EN

	            // Create KeyInfo structure - START

	            Element keyInfoElem = doc.createElementNS(WSConstants.SIG_NS,
	                    "KeyInfo");
	            ((OMElement) encrKeyElem).declareNamespace(WSConstants.SIG_NS,
	                    WSConstants.SIG_PREFIX);
	            ((OMElement) encrKeyElem).declareNamespace(WSConstants.ENC_NS,
	                    WSConstants.ENC_PREFIX);
	            keyInfoElem.appendChild(encrKeyElem);

	            // Create KeyInfo structure - END

	            SAMLStatement[] statements = createSAMLStatements(ipData, keyInfoElem);

	            // Assertion
	            SAMLAssertion assertion = new SAMLAssertion(ISSUER_SELF, notBefore,
	                    notAfter, null, null, Arrays.asList(statements));
	            
	            //Sign the assertion
	            X509Certificate[] issuerCerts = crypto.getCertificates("ip");

	            String sigAlgo = XMLSignature.ALGO_ID_SIGNATURE_RSA;
	            String pubKeyAlgo = issuerCerts[0].getPublicKey().getAlgorithm();
	            if (pubKeyAlgo.equalsIgnoreCase("DSA")) {
	                sigAlgo = XMLSignature.ALGO_ID_SIGNATURE_DSA;
	            }
	            Key issuerPK = crypto.getPrivateKey("ip", "password");
	            assertion.sign(sigAlgo, issuerPK, Arrays.asList(issuerCerts));

	            // This will be required to send back to the requester in the RSTR
	            byte[] key = encryptedKey.getEphemeralKey();
	            OMElement rstrElem = createRSTR(rahasData, notBefore, notAfter,
	    			env, doc, assertion, key);
	            
	            createDisplayToken(rstrElem, ipData);
	     
	            return env;

	        } catch (Exception e) {
	            throw new TrustException("errorLoadingConfigFile", e);
	        }
	    }
	    
	    

	    /**
		 * 
		 * @param data
		 * @param notBefore
		 * @param notAfter
		 * @param env
		 * @param doc
		 * @param assertion
		 * @param key
		 * @return
		 * @throws TrustException
		 * @throws SAMLException
		 */
		public OMElement createRSTR(RahasData data, Date notBefore, Date notAfter,
				SOAPEnvelope env, Document doc, SAMLAssertion assertion, byte[] key)
		throws TrustException, SAMLException {

			int wstVersion = data.getVersion();
			MessageContext inMsgCtx = data.getInMessageContext();

			OMElement rstrElem = TrustUtil
			.createRequestSecurityTokenResponseElement(wstVersion, 
					env.getBody());
			TrustUtil.createTokenTypeElement(wstVersion, rstrElem).setText(
					RahasConstants.TOK_TYPE_SAML_10);

			TrustUtil.createRequestedAttachedRef(wstVersion, rstrElem, "#"
					+ assertion.getId(), RahasConstants.TOK_TYPE_SAML_10);

			TrustUtil.createRequestedUnattachedRef(wstVersion, rstrElem, assertion
					.getId(), RahasConstants.TOK_TYPE_SAML_10);

			TrustUtil.createAppliesToElement(rstrElem, data.getAppliesToAddress(),
					data.getAddressingNs());

			// Use GMT time in milliseconds
			DateFormat zulu = new XmlSchemaDateFormat();

			// Add the Lifetime element
			TrustUtil.createLifetimeElement(wstVersion, rstrElem, zulu
					.format(notBefore), zulu.format(notAfter));

			OMElement reqSecTokenElem = TrustUtil
			.createRequestedSecurityTokenElement(wstVersion, rstrElem);

			Node tempNode = assertion.toDOM();
			reqSecTokenElem.addChild((OMNode) ((Element) rstrElem)
					.getOwnerDocument().importNode(tempNode, true));

			// Store the Token
			Token assertionToken = new Token(assertion.getId(),
					(OMElement) assertion.toDOM(), notBefore, notAfter);

			// At this point we definitely have the secret
			// Otherwise it should fail with an exception earlier
			assertionToken.setSecret(data.getEphmeralKey());
			TrustUtil.getTokenStore(inMsgCtx).add(assertionToken);

			// Creating the ReqProoftoken - START
			OMElement reqProofTokElem = TrustUtil
			.createRequestedProofTokenElement(wstVersion, rstrElem);

			WSSecEncryptedKey encrKeyBuilder = new WSSecEncryptedKey();
			encrKeyBuilder
			.setKeyIdentifierType(WSConstants.THUMBPRINT_IDENTIFIER);
			try {
				encrKeyBuilder.setUseThisCert(data.getClientCert());
				encrKeyBuilder.setEphemeralKey(key);
				encrKeyBuilder.prepare(doc, null);
			} catch (WSSecurityException e) {
				throw new TrustException(
						"errorInBuildingTheEncryptedKeyForPrincipal",
						new String[] { data.getClientCert().getSubjectDN()
								.getName() });
			}
			Element encryptedKeyElem = encrKeyBuilder.getEncryptedKeyElement();
			Element bstElem = encrKeyBuilder.getBinarySecurityTokenElement();
			if (bstElem != null) {
				reqProofTokElem.addChild((OMElement) bstElem);
			}

			reqProofTokElem.addChild((OMElement) encryptedKeyElem);
			// Creating the ReqProoftoken - END

			return rstrElem;
		}

		/**
		 * 
		 * @param rahasData
		 * @param ipData
		 * @return
		 */
		private OMElement createDisplayToken(OMElement rstrElem, IdentityProviderData ipData){
			
			Map requestedClaims = ipData.getRequestedClaims();
			
			if(requestedClaims.isEmpty())
				return null;
			
			OMElement rdt =  IdentityProviderUtil.createRequestedDisplayToken(rstrElem, ipData);
			OMElement displayToken = IdentityProviderUtil.createDisplayToken(rdt, ipData);
			
			Iterator ite = requestedClaims.values().iterator();
			while(ite.hasNext()){
				Claim claims  = (Claim) ite.next();
				IdentityProviderUtil.createDisplayClaim(displayToken, ipData.getDisplayName(claims.URI), claims.value);
			}
			
			return rdt;
		}
	    /**
	     * @param givenName
	     * @param surname
	     * @param email
	     * @param keyInfoElem
	     * @return
	     * @throws SAMLException
	     */
	    private SAMLStatement[] createSAMLStatements(IdentityProviderData ipData, Element keyInfoElem)
	            throws SAMLException, IdentityProviderException {

	        // The SAMLSubject

	        String[] confirmationMethods = new String[] { SAMLSubject.CONF_HOLDER_KEY };

	        SAMLSubject subject = new SAMLSubject(null, Arrays
	                .asList(confirmationMethods), null, keyInfoElem);
	        
	        
	        Map mapClaims = ipData.getRequestedClaims();
	        	     
	        List assertionList = new ArrayList();
	        Iterator ite = mapClaims.values().iterator();
	       
	        while(ite.hasNext()){
	        	Claim claim = (Claim)ite.next();
	        	List list = Arrays.asList(new String[] {claim.value}); 
	        	String displayTag = ipData.getDisplayName(claim.URI);
	        	if((displayTag == null) && (claim.bOptional==false)){
	        		throw new IdentityProviderException("Do not understand must understand. Unknown claim URI");
	        	}
	        	SAMLAttribute attr= new SAMLAttribute(claim.URI, displayTag ,
		                null, -1, list);
	        	assertionList.add(attr);
	        	
	    
	        }
	         SAMLAttribute[] attrs = (SAMLAttribute[]) assertionList.toArray();
	   
	        // AttributeStatement
	        SAMLAttributeStatement attrStmt = new SAMLAttributeStatement(subject,
	                Arrays.asList(attrs));

	        SAMLStatement[] statements = { attrStmt };
	        return statements;
	    }

	    public void setConfigurationElement(OMElement configElement) {
	    }

	    public void setConfigurationFile(String configFile) {
	        this.confileFilePath = configFile;
	    }

	    public void setConfigurationParamName(String configParamName) {

	    }
	    
	    /**
	     * This method will get a user from UserManager service.
	     * @param username
	     * @return
	     */
	    public IPUser getUser(String username){
	    	IPUser user = null;
	    	//TODO:: put these things into a configuration file
	    	GenericJNDIRealm realm = new GenericJNDIRealm();
			
			realm.setConnectionName("cn=root,dc=wso2,dc=com");
			realm.setConnectionPassword("secret");
			realm.setConnectionURL("ldap://10.100.1.138");
			
			realm.setDigestAlgo("MD5");
			realm.setPasswordAttrId("userPassword");

			Map map = new HashMap();

			map.put("password", "userPassword");
			map.put("name", "givenName");
			
			realm.setUserPattern("uid={0},ou=People,dc=wso2,dc=com");
			realm.configureUserBaseDriver(map, "org.wso2.solutions.identity.sts.users.IPUser");
			
	    	try {
				user = (IPUser) realm.getUser(username);
	    	} catch (PrincipalNotFoundException e) {
				e.printStackTrace();
			}
	      	return user;
	    }
	    
	    

	}



