#include <string.h>
#include <stdio.h>
#include <axis2_client.h>
#include <axiom_document.h>
#include <axiom_element.h>
#include <axiom_node.h>
#include <axiom_stax_builder.h>
#include <axiom_xml_reader.h>
#include <axis2_util.h>
#include <axis2_env.h>
#include <oxs_ctx.h>
#include <oxs_utility.h>
#include <oxs_key.h>
#include <oxs_asym_ctx.h>
#include <oxs_axiom.h>
#include <oxs_token_encryption_method.h>
#include "httpd.h"
#include "mod_cspace.h"

/*typedef struct {
 *       const axis2_env_t *env;
 *       axis2_allocator_t *allocator;
 *       axiom_document_t *doc;
 *       axiom_xml_reader_t *reader;
 *       axiom_stax_builder_t *builder;
 *       axiom_node_t *node;
 *       char *key_file;
 *
 *       axis2_svc_client_t *svc_client;
 *       axis2_options_t *options;
 *       axis2_endpoint_ref_t *epr;
 * } process_ctx_t;
 */

void * process_ctx_malloc(axis2_allocator_t *alloc, size_t sz) {
    return apr_palloc((apr_pool_t *)(alloc->current_pool), sz);
}

/*TODO: Not Impld, we don't need this*/
void * process_ctx_realloc(axis2_allocator_t *alloc, void *ptr, size_t sz) {
    return NULL;
}

/*handled by apr pools*/
void process_ctx_free(axis2_allocator_t *alloc, void *ptr) {
}

process_ctx_t* cspace_process_ctx_create(void *pool) {
    /* TODO: we need ctx, alloc and the env to be allocated using the global pool
     * r->server->process->pool?? 
     */

    process_ctx_t *ctx;
    ctx = (process_ctx_t *)apr_palloc((apr_pool_t*)pool, sizeof(process_ctx_t));
    if (!ctx)
        return NULL;
    ctx->allocator = (axis2_allocator_t *)apr_palloc((apr_pool_t*)pool, 
            sizeof(axis2_allocator_t));
    if (!ctx->allocator)
        return NULL;
   
    ctx->allocator->malloc_fn = process_ctx_malloc;
    ctx->allocator->realloc = process_ctx_realloc;
    ctx->allocator->free_fn = process_ctx_free;
    ctx->allocator->local_pool = pool;
    ctx->allocator->global_pool = pool;
    ctx->allocator->current_pool = pool;

    ctx->env = axis2_env_create(ctx->allocator);
    if (!ctx->env) {
        AXIS2_FREE(ctx->allocator, ctx->allocator);
        ctx->allocator = NULL;
        return NULL;
    }
    
    return ctx;
}

static axiom_node_t* make_om_node(process_ctx_t *ctx, const axis2_char_t* req) {
    /*TODO: find the objs that need not be in ctx and remove them from ctx*/
    ctx->reader = axiom_xml_reader_create_for_memory(ctx->env, (void *)req, 
            strlen(req), "UTF-8", AXIS2_XML_PARSER_TYPE_BUFFER);
    if (!ctx->reader)
        return NULL;

    if (!(ctx->builder = axiom_stax_builder_create(ctx->env, ctx->reader)))
        return NULL;

    if (!(ctx->doc = axiom_document_create(ctx->env, NULL, ctx->builder)))
        return NULL;

    if (!(ctx->node = axiom_document_build_all(ctx->doc, ctx->env)))
        return NULL;

    return ctx->node;
}

#define XML_DSIG_NS "http://www.w3.org/2000/09/xmldsig#"
#define XML_ENC_NS "http://www.w3.org/2001/04/xmlenc#"
#define ENC_KEY  "EncryptedKey"
#define ENC_MTD  "EncryptionMethod"
#define KEY_INFO "KeyInfo"

#define find_child(node, name, uri, pfx) \
            (node) ? oxs_axiom_get_first_child_node_by_name \
                        (ctx->env, (node), (name), (uri), (pfx)) : NULL

static int get_session_key(process_ctx_t *ctx, axiom_node_t *node, 
        oxs_key_t **sess_key) {
    axiom_node_t *enc_key_node = NULL;
    axiom_node_t *enc_mtd_node = NULL;
    oxs_asym_ctx_t *asym_ctx = NULL;
    axis2_char_t *enc_algo = NULL;
    int rv;
    
    axiom_node_t *tempnode = find_child(node, KEY_INFO, XML_DSIG_NS, NULL);

    enc_key_node = find_child(tempnode, ENC_KEY, XML_ENC_NS, "e");
    if (!enc_key_node) return FAIL;

    enc_mtd_node = find_child(enc_key_node, ENC_MTD, XML_ENC_NS, "e");
    if (!enc_mtd_node) return FAIL;
    
    enc_algo = oxs_token_get_encryption_method(ctx->env, enc_mtd_node);
    if (!enc_algo) return FAIL;
    
    asym_ctx = oxs_asym_ctx_create(ctx->env);
    if (!asym_ctx) return FAIL;

    oxs_asym_ctx_set_file_name(asym_ctx, ctx->env, ctx->key_file);
    oxs_asym_ctx_set_format(asym_ctx, ctx->env, 
            oxs_util_get_format_by_file_extension(ctx->env, ctx->key_file));
    oxs_asym_ctx_set_operation(asym_ctx, ctx->env, 
            OXS_ASYM_CTX_OPERATION_PRV_DECRYPT); oxs_asym_ctx_set_password(asym_ctx, ctx->env, "1234"); /*TODO:MAGIC*/
    oxs_asym_ctx_set_algorithm(asym_ctx, ctx->env, enc_algo);

    *sess_key = oxs_key_create(ctx->env);
    if (!(*sess_key)) return FAIL;

    rv = oxs_xml_enc_decrypt_key(ctx->env, asym_ctx, enc_key_node, 
            enc_key_node, *sess_key);

    return rv;
}

static int get_decrypted_node(process_ctx_t *ctx, axiom_node_t *node,
        oxs_key_t *key, axiom_node_t **dec_node) {
    oxs_ctx_t *sym_ctx = NULL;
    int rv;   

    sym_ctx = oxs_ctx_create(ctx->env);
    if (!sym_ctx) return FAIL;

    oxs_ctx_set_key(sym_ctx, ctx->env, key);
    rv = oxs_xml_enc_decrypt_node(ctx->env, sym_ctx, node, dec_node);
    
    return rv;
}

static axiom_node_t* decrypt_node(process_ctx_t *ctx, axiom_node_t *node) {
    oxs_key_t *sess_key = NULL;
    axiom_node_t *dec_node = NULL;    
    
    if (!get_session_key(ctx, node, &sess_key))
        return NULL;
    
    if (!get_decrypted_node(ctx, node, sess_key, &dec_node))
        return NULL;
    
#ifdef CSPACE_DEBUG
    printf("\ndec:\n%s\n\n",axiom_node_to_string(dec_node, ctx->env));
#endif

    return dec_node;
}

#define AXIS2C_HOME "AXIS2C_HOME"

int init_axis2_client(process_ctx_t *ctx, char *epr) {
    char* client_home = NULL;

    ctx->epr = axis2_endpoint_ref_create(ctx->env, epr);
    if (!ctx->epr) return FAIL;
    
    ctx->options = axis2_options_create(ctx->env);
    if (!ctx->options) return FAIL;
    
    axis2_options_set_to(ctx->options, ctx->env, ctx->epr);
    axis2_options_set_action(ctx->options, ctx->env, CSPACE_TVS_ACTION);

    client_home = AXIS2_GETENV(AXIS2C_HOME);
    if (!client_home) return FAIL;
    
    ctx->svc_client = axis2_svc_client_create(ctx->env, client_home);
    if (!ctx->svc_client) return FAIL;

    axis2_svc_client_set_options(ctx->svc_client, ctx->env, ctx->options);

    axis2_svc_client_engage_module(ctx->svc_client, ctx->env,
            AXIS2_MODULE_ADDRESSING);

    return SUCC;
}

#define invoke_tvs(ctx, node) axis2_svc_client_send_receive( \
            (ctx)->svc_client, (ctx)->env, (node))

#define CSPACE_TVS_RES_ROOT "VerifierResponse"
#define CSPACE_TVS_RES_PROPERTY "Property"
#define CSPACE_TVS_RES_PROPERTY_ATTR "Name"
#define CSPACE_TVS_RES_STATE_ATTR "State"

#define CSPACE_TVS_RES_STATE_AUTHED "authenticated"
#define CSPACE_TVS_RES_STATE_PENDING "newUserConfirmationPending"

/* sample response of the token verifier
 *
 * <rp:VerifierResponse xmlns:rp="http://ns.wso2.org/relyingparty">
 *     <Property Name="foo">bar</Property>
 *     <Property Name="foo1">bar1</Property>
 *     <Property Name="foo2">bar2</Property>
 *     <Property Name="foo3">bar3</Property>
 * </rp:VerifierResponse>
 */

#define TEST_RESPONSE "<rp:VerifierResponse State=\"newUserConfirmationPending\" xmlns:rp=\"http://ns.wso2.org/relyingparty\"><rp:Property Name=\"givenname\">Ruchith</rp:Property><rp:Property Name=\"surname\">Fernando</rp:Property><rp:Property Name=\"privatepersonalidentifier\">5j+05kKsBEQeJOk4NhaY0e4BxxnBnYWwDlsiFWd/550=</rp:Property><rp:Property Name=\"emailaddress\">r@ruchith.org</rp:Property></rp:VerifierResponse>"

#define verify_element_name(ctx, ele, name) \
    (strcmp(axiom_element_get_localname((ele), (ctx)->env), (name)) == 0 )

static int process_response(process_ctx_t *ctx, axiom_node_t *node, 
        void* ret_hash, hash_set_cb_t cb) {
    axiom_element_t *vr_ele;
    axiom_node_t *prop_node;
    axiom_element_t *prop_ele;
    char *attr_val;
    char *text_val;
    int state = FAIL;

    axiom_child_element_iterator_t *ele_itr;

#ifdef CSPACE_DEBUG
    printf(AXIOM_NODE_TO_STRING(node, ctx->env));
#endif

    vr_ele = AXIOM_NODE_GET_DATA_ELEMENT(node, ctx->env);
    if (!vr_ele) 
        return FAIL; /*Response not understood*/
    if (!(verify_element_name(ctx, vr_ele, CSPACE_TVS_RES_ROOT)))
        return FAIL; /*Response not understood*/
    
    attr_val = axiom_element_get_attribute_value_by_name(vr_ele, ctx->env,
            CSPACE_TVS_RES_STATE_ATTR);
    if (strcmp(attr_val, CSPACE_TVS_RES_STATE_AUTHED) == 0) {
        state = CSPACE_STATE_AUTHED;
    } else if (strcmp(attr_val, CSPACE_TVS_RES_STATE_PENDING) == 0) {
        state = CSPACE_STATE_PENDING;
    }
    (*cb)("cardspace_auth_state", attr_val, ret_hash);

    attr_val = NULL;

    ele_itr = axiom_element_get_child_elements(vr_ele, ctx->env, node);
    
    if (ele_itr) {
        while (AXIOM_CHILD_ELEMENT_ITERATOR_HAS_NEXT(ele_itr, ctx->env)) {
            prop_node = AXIOM_CHILD_ELEMENT_ITERATOR_NEXT(ele_itr, ctx->env);

            if (prop_node) {
                prop_ele = AXIOM_NODE_GET_DATA_ELEMENT(prop_node, ctx->env);
                if (prop_ele) {
                    attr_val = axiom_element_get_attribute_value_by_name(
                            prop_ele, ctx->env, CSPACE_TVS_RES_PROPERTY_ATTR);
                    
                    text_val = axiom_element_get_text(prop_ele, ctx->env,
                            prop_node);

                    if (attr_val && text_val) {
                        (*cb)(attr_val, text_val, ret_hash);
                    }
                }
            }
        }
    }
    
/*#ifdef CSPACE_DEBUG
    (*cb)("TestKey1", "Val1", ret_hash);
#endif*/

    return state;
}

#define CSPACE_RP_NAMESPACE "http://ns.wso2.org/relyingparty"
#define CSPACE_TVS_REQ_ROOT "VerifierRequest"
#define CSPACE_TVS_REQ_TOKEN "Token"
#define CSPACE_TVS_REQ_ISSUER "Issuer"

static axiom_node_t* create_tvs_node(process_ctx_t *ctx, 
        axiom_node_t *saml_node, const char *issuer) {
    axiom_node_t *vr_node = NULL;
    
    axiom_node_t *node, *txt_node;
    axiom_element_t *ele, *vr_ele;
    axiom_namespace_t *ns;
    axiom_text_t *issuer_txt;

    ns = axiom_namespace_create(ctx->env, CSPACE_RP_NAMESPACE, "rp");
    vr_ele = axiom_element_create(ctx->env, NULL, CSPACE_TVS_REQ_ROOT, ns, &vr_node);
    ele = axiom_element_create(ctx->env, vr_node, CSPACE_TVS_REQ_TOKEN, ns, &node);
    axiom_node_add_child(node, ctx->env, saml_node);
    ele = axiom_element_create(ctx->env, vr_node, CSPACE_TVS_REQ_ISSUER, ns, &node);
    issuer_txt = axiom_text_create(ctx->env, node, issuer, &txt_node);
    return vr_node;
}

int cspace_process_request(process_ctx_t *ctx, char *req, char* key_file,
        char* epr, void *ret_hash, hash_set_cb_t cb) {
    axiom_node_t *root_node, *ret_node;
    axiom_node_t *dec_saml_node;
    int state = FAIL;

    ctx->key_file = key_file; /*no need to strcpy we just use it*/

    req = strstr((const char *)req, CSPACE_XML_TOKEN) /*xmlToken=*/
                + strlen(CSPACE_XML_TOKEN);

    root_node = make_om_node(ctx, req);
    if (!(root_node)) return FAIL;

#ifdef CSPACE_DEBUG
    printf("\nserialized:\n %s \n\n",
            AXIOM_NODE_TO_STRING(root_node, ctx->env));
#endif

    dec_saml_node = decrypt_node(ctx, root_node);
    if (!dec_saml_node) return FAIL;
    
    init_axis2_client(ctx, epr);

/*#ifdef CSPACE_DEBUG
    ret_node = make_om_node(ctx, TEST_RESPONSE);
#else */
    dec_saml_node = create_tvs_node(ctx, dec_saml_node, "self");
    
    ret_node = invoke_tvs(ctx, dec_saml_node);
    
    if (!ret_node) {
        printf("\n\nTVS INVOKE FAILED!!!!!\n");
        return FAIL;
    }
/*#endif */
    
#ifdef CSPACE_DEBUG
    printf("received node: %s\n\n", AXIOM_NODE_TO_STRING(ret_node, ctx->env));
#endif 
    state = process_response(ctx, ret_node, ret_hash, cb);
   
    /*process_ctx_free(ctx); no free, using apr pools */

    return state;
}
