/*
 * Copyright 2004,2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.relyingparty;

import org.apache.axiom.om.OMAttribute;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;
import org.apache.axis2.util.XMLUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.security.WSSConfig;
import org.w3c.dom.Element;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.xml.namespace.QName;

import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.util.Iterator;


public class RPFilter implements Filter {
    
    public static Log log = LogFactory.getLog(RPFilter.class);

    static {
        WSSConfig.getDefaultWSConfig();
    }
    
    FilterConfig filterConfig;
    
    public void doFilter(ServletRequest request, ServletResponse response,
            FilterChain chain) throws IOException, ServletException {
        try {
            
            String path = ((HttpServletRequest) request).getRequestURI();
            log.debug("RequestURI : " + path);
            if (path.indexOf("config") < 0) {
                
                String configLocationParam = filterConfig.getInitParameter("config-file");
                
                RPConfig config = new RPConfig(configLocationParam);

                String infoCardSignin = request.getParameter("InfoCardSignin");
                log.debug("InfoCardSignin : " + infoCardSignin);
                if (infoCardSignin != null && "Log in".equals(infoCardSignin)) {
                    log.debug("InfoCardSignin=Log in");
                    String xmlToken = request.getParameter("xmlToken");
                    if (xmlToken != null) {

                        PrivateKey privKey = getPrivateKey(config);
                        TokenVerifier verifier = new TokenVerifier();
                        Element plainTokenElem = verifier.decryptToken(xmlToken, privKey);
                        OMElement verificationResponseElem = verifyToken(plainTokenElem, config);
                        
                        OMAttribute attr = verificationResponseElem.getAttribute(new QName(RPConstants.ATTR_STATE));
                        request.setAttribute(RPConstants.ATTR_STATE, attr.getAttributeValue());
                        Iterator propertyElems = verificationResponseElem.getChildrenWithName(new QName(RPConstants.NS, RPConstants.LN_PROPERTY, RPConstants.PREFIX));
                        while (propertyElems.hasNext()) {
                            OMElement property = (OMElement) propertyElems.next();
                            OMAttribute nameAttr = property.getAttribute(new QName(RPConstants.ATTR_NAME));
                            request.setAttribute(nameAttr.getAttributeValue(), property.getText());
                        }
                        
                    } else {
                        reportError(response, "Authentication failure : xmlToken missing", null);
                    }
                }
            }
            chain.doFilter(request, response);
        } catch (Exception e) {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            PrintStream ps = new PrintStream(baos);
            e.printStackTrace(ps);
            reportError(response, e.getMessage(), new String(baos.toByteArray()));
        }
    }

    /**
     * Access the verifier service and obtain the verification response
     * @param plainTokenElem
     * @param config
     * @return
     */
    private OMElement verifyToken(Element plainTokenElem, RPConfig config) throws Exception {
        OMElement elem = XMLUtils.toOM(plainTokenElem);
        OMFactory fac = elem.getOMFactory();
        OMElement reqElem = fac.createOMElement(new QName(RPConstants.NS, RPConstants.LN_REQ, RPConstants.PREFIX));
        OMElement tokenElem = fac.createOMElement(new QName(RPConstants.NS, RPConstants.LN_TOKEN, RPConstants.PREFIX), reqElem);
        tokenElem.addChild(elem);
        OMElement issuerElem = fac.createOMElement(new QName(RPConstants.NS, RPConstants.LN_ISSUER, RPConstants.PREFIX), reqElem);
        issuerElem.setText(RPConstants.ISSUER_SELF);

        ServiceClient client = new ServiceClient();
        
        Options options = new Options();
        options.setAction("urn:verify");
        options.setTo(new EndpointReference(config.getVerifierURL()));
        
        client.setOptions(options);
        return client.sendReceive(reqElem);
        
    }

    /**
     * @return
     */
    private PrivateKey getPrivateKey(RPConfig config) throws Exception {
        KeyStore store = KeyStore.getInstance(config.getServiceStoreType());

        store.load(new FileInputStream(config.getServiceKeystore()), config
                .getServiceStorePass().toCharArray());

        return (PrivateKey) store.getKey(config.getServiceKeyAlias(), config
                .getServiceKeyPass().toCharArray());
    }

    private void reportError(ServletResponse response, String error, String trace)
            throws IOException {

        PrintWriter out = response.getWriter();

        response.setContentType("text/html");
        out.println("<html><body>");
        out.println("<H1>" + error + "</H1><BR/>");
        if(trace != null) {
            out.println("<p>" + trace + "</p>");
        }
        out.println("<hr/><a href=\"http://www.wso2.org/projects/identity\">WSO2 Identity Solution</a>");
        out.println("</body></html>");
        response.flushBuffer();
    }

    public void destroy() {
    }

    public void init(FilterConfig config) throws ServletException {
        this.filterConfig = config;
    }

}