/*
 * Copyright 2004,2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.relyingparty;

import org.apache.ws.security.WSConstants;
import org.apache.ws.security.processor.EncryptedKeyProcessor;
import org.apache.ws.security.util.WSSecurityUtil;
import org.apache.xml.security.Init;
import org.apache.xml.security.encryption.XMLCipher;
import org.apache.xml.security.utils.EncryptionConstants;
import org.opensaml.SAMLAssertion;
import org.opensaml.SAMLAttribute;
import org.opensaml.SAMLAttributeStatement;
import org.opensaml.SAMLException;
import org.opensaml.SAMLStatement;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import javax.crypto.SecretKey;
import javax.xml.parsers.DocumentBuilderFactory;

import java.io.ByteArrayInputStream;
import java.security.PrivateKey;
import java.util.Hashtable;
import java.util.Iterator;

public class TokenVerifier {

    private Hashtable attributeTable = new Hashtable();

    static {
        Init.init();
    }

    public Element decryptToken(String token, PrivateKey serviceKey)
            throws RelyingPartyException {

        try {
            ByteArrayInputStream bais = new ByteArrayInputStream(token
                    .getBytes());
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            dbf.setNamespaceAware(true);
            Document doc = dbf.newDocumentBuilder().parse(bais);
            Element encryptedTokenElem = doc.getDocumentElement();

            // Decrypt element
            return this.decryptElement(serviceKey,
                    encryptedTokenElem);

        } catch (Exception e) {
            throw new RelyingPartyException(
                    RelyingPartyException.VERIFICATION_FAILURE, e);
        }

    }

    /**
     * @param decryptedElem
     * @return
     * @throws SAMLException
     */
    public boolean verifyDecryptedToken(Element decryptedElem)
            throws RelyingPartyException {
        try {
            // Convert to SAML
            SAMLAssertion assertion = new SAMLAssertion(decryptedElem);
            if (assertion.isSigned()) {
                assertion.verify();
                // TODO: Now check for the trusted cert
            }

            Iterator statements = assertion.getStatements();
            while (statements.hasNext()) {
                SAMLStatement stmt = (SAMLStatement) statements.next();
                if (stmt instanceof SAMLAttributeStatement) {
                    Iterator attrs = ((SAMLAttributeStatement) stmt)
                            .getAttributes();
                    while (attrs.hasNext()) {
                        SAMLAttribute attr = (SAMLAttribute) attrs.next();
                        // TODO: There can be multiple values .. get all of them
                        // :-)
                        this.attributeTable.put(attr.getName(), attr
                                .getValues().next());
                    }
                }

            }
        } catch (SAMLException e) {
            throw new RelyingPartyException(
                    RelyingPartyException.VERIFICATION_FAILURE, e);
        }
        // If we reach this point ... everythign is fine :D
        return true;
    }

    private Element decryptElement(PrivateKey privKey, Element encryptedToken)
            throws Exception {

        Element kiElem = (Element) encryptedToken.getElementsByTagNameNS(
                WSConstants.SIG_NS, "KeyInfo").item(0);
        Element encrKeyElem = (Element) kiElem.getElementsByTagNameNS(
                WSConstants.ENC_NS, EncryptionConstants._TAG_ENCRYPTEDKEY)
                .item(0);

        EncryptedKeyProcessor encrKeyProcessor = new EncryptedKeyProcessor();
        encrKeyProcessor.handleEncryptedKey(encrKeyElem, privKey);

        SecretKey secretKey = WSSecurityUtil.prepareSecretKey(
                EncryptionConstants.ALGO_ID_BLOCKCIPHER_AES128,
                encrKeyProcessor.getDecryptedBytes());

        XMLCipher cipher = XMLCipher.getInstance();
        cipher.init(XMLCipher.DECRYPT_MODE, secretKey);

        Document doc = cipher.doFinal(encryptedToken.getOwnerDocument(),
                encryptedToken);
        return doc.getDocumentElement();
    }

    public Hashtable getAttributeTable() {
        return attributeTable;
    }

}