/*
 * Copyright 2004,2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.relyingparty;

import org.apache.axiom.om.util.Base64;
import org.apache.axis2.context.MessageContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.usermanager.PrincipalCreationException;
import org.wso2.usermanager.PrincipalNotFoundException;
import org.wso2.usermanager.UserManException;
import org.wso2.usermanager.elements.User;
import org.wso2.usermanager.realm.generic.GenericJDBCRealm;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.Driver;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.Properties;
import java.util.Random;

public class Users {

    public static Log log = LogFactory.getLog(Users.class);

    public final static int USER_NOT_AVAILABLE = 1;

    public final static int USER_REGISTERED = 2;

    public final static int USER_CONFIRMATION_PENDING = 3;

    public static GenericJDBCRealm realm = null;

    private Users() {
    }

    private static Users users = new Users();

    // private Hashtable userList = new Hashtable();

    public static final String DRIVER = "org.apache.derby.jdbc.EmbeddedDriver";

    public static final String PROTOCOL = "jdbc:derby:";

    public static final String WSAS_HOME = "WSO2WSAS_HOME";

    public static final String USER_CLASS = "org.wso2.solutions.identity.relyingparty.RPSelfIssuedUser";

    public static String dbPath = null;

    public static Users getInstance() {

        return users;
    }

    static {
        /*
         * Connecting to the database.
         */
    	
    	SetupDB();
    	
        realm = new GenericJDBCRealm();

        if (dbPath == null) {
            String wsasHome = System.getProperty(WSAS_HOME);
            if (wsasHome == null) {
                wsasHome = "./";
            } else {
                wsasHome = wsasHome + "/";
            }
            realm.setConnectionURL(PROTOCOL + wsasHome
                    + "UserDB/DBSelfIssuedUsers;");
        } else {
            realm.setConnectionURL(PROTOCOL + dbPath
                    + "UserDB/DBSelfIssuedUsers;");
        }

        realm.setDriverName(DRIVER);

        realm.setUserTable("Users");
        realm.setUserNameCol("PPID");

        Map map = new HashMap();
        map.put("name", "PPID");
        map.put("confirmString", "CONFIRM");
        map.put("status", "STATUS");

        realm.configureUserBaseDriver(map, USER_CLASS);

        try {
            realm.open();
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }

    public int handle(Hashtable attributeTable, String epr)
            throws RelyingPartyException {
        String ppid = (String) attributeTable.get(RPConstants.SAML_ATTR_PPID);

        int userState = this.getState(ppid);
        if (userState == Users.USER_NOT_AVAILABLE) {
            String confString = this.addUser(ppid);
            sendConfirmationMail((String) attributeTable
                    .get(RPConstants.SAML_ATTR_EMALADDR),
                    (String) attributeTable
                            .get(RPConstants.SAML_ATTR_GIVENNAME), confString,
                    epr);
        }
        
        return userState;
    }

    private void sendConfirmationMail(String emailAddr, String name,
            String confString, String epr) throws RelyingPartyException {
        MessageContext ctx = MessageContext.getCurrentMessageContext();

        Properties props = new Properties();

        props.put(RPConstants.SERVICE_CONFIG_SMTP_HOST, (String) ctx
                .getParameter(RPConstants.SERVICE_CONFIG_SMTP_HOST).getValue());
        props.put(RPConstants.SERVICE_CONFIG_SMTP_PORT, (String) ctx
                .getParameter(RPConstants.SERVICE_CONFIG_SMTP_PORT).getValue());
        Session session = Session.getDefaultInstance(props, null);

        try {
            // Construct the message
            Message msg = new MimeMessage(session);
            msg.setFrom(new InternetAddress((String) ctx.getParameter(
                    RPConstants.SERVICE_CONFIG_FROM_ADDRESS).getValue()));
            msg.setRecipient(Message.RecipientType.TO, new InternetAddress(
                    emailAddr));
            msg.setSubject((String) ctx.getParameter(
                    RPConstants.SERVICE_CONFIG_FROM_SUBJECT).getValue());
            msg.setText(getResponseMessage(confString, epr));

            // Send the message
            Transport.send(msg);
        } catch (AddressException e) {
            throw new RelyingPartyException(RelyingPartyException.DEFAULT, e);
        } catch (MessagingException e) {
            throw new RelyingPartyException(RelyingPartyException.DEFAULT, e);
        }

    }

    private String getResponseMessage(String confString, String epr) {
        // find out the REST epr of the user reg service

        return "Point your browser to the following address : \n " + epr + confString;
    }

    public String addUser(String ppid) throws RelyingPartyException {
        try {
            Random random = new Random();
            byte[] temp = new byte[16];
            random.nextBytes(temp);
            String confString = Base64.encode(temp);
            RPSelfIssuedUser user = new RPSelfIssuedUser();
            user.setConfirmString(confString);
            user.setName(ppid);
            user.setStatus(USER_CONFIRMATION_PENDING);
            realm.createUser(user);
            return confString;
        } catch (PrincipalCreationException e) {
            log.error(e.getMessage(), e);
            throw new RelyingPartyException(RelyingPartyException.DEFAULT, e);
        }

        /*
         * Random random = new Random(); byte[] temp = new byte[16];
         * random.nextBytes(temp); String confString = Base64.encode(temp); User
         * user = new User(confString); userList.put(ppid, user);
         * 
         * return confString;
         */
    }

    public boolean confirm(String confString) throws RelyingPartyException {
        if (confString == null)
            return false;

        String stmt = "select * from Users where CONFIRM = '" + confString
                + "'";

        try {
            User users[] = realm.selectUsers(stmt);
            if (users != null) {
                if (users.length == 1) {
                    RPSelfIssuedUser user = (RPSelfIssuedUser) users[0];
                    if (confString.equals(user.getConfirmString())) {
                        user.setStatus(USER_REGISTERED);
                        realm.updateUser(user);
                        return true;
                    }
                } else {
                    log.error("Couple of users have same random number. Please check");
                }
            }
        } catch (UserManException e) {
            log.error(e.getMessage(), e);
            throw new RelyingPartyException(RelyingPartyException.DEFAULT, e);
        } catch (PrincipalNotFoundException e) {
            log.error(e.getMessage(), e);
            throw new RelyingPartyException(RelyingPartyException.DEFAULT, e);
        }
        return false;

        /*
         * Enumeration usersEnum = userList.elements(); while
         * (usersEnum.hasMoreElements()) { User user =
         * (User)usersEnum.nextElement(); if(user.confString.equals(confString)) {
         * user.confirmed = true; return true; } } //If the user was not found
         * return false;
         */
    }

    public int getState(String ppid) throws RelyingPartyException {
        int returnStatus = USER_NOT_AVAILABLE;
        try {
            RPSelfIssuedUser user = (RPSelfIssuedUser) realm.getUser(ppid);
            if(user != null) {
                returnStatus = user.getStatus();
            }  
        } catch (PrincipalNotFoundException e) {
            log.error(e.getMessage(), e);
            throw new RelyingPartyException(RelyingPartyException.DEFAULT, e);
        }
        return returnStatus;

    }
    
    public static void SetupDB(){

		try{
			
				Class clazz = Class.forName(DRIVER);
				Driver driver = (Driver) clazz.newInstance();
				
				Properties props = new Properties();
				
				String connectionURL = null;
				 if (dbPath == null) {
			            String wsasHome = System.getProperty(WSAS_HOME);
			            if (wsasHome == null) {
			                wsasHome = "./";
			            } else {
			                wsasHome = wsasHome + "/";
			            }
			            connectionURL = PROTOCOL + wsasHome
			                    + "UserDB/DBSelfIssuedUsers;create=true";
			        } else {
			            realm.setConnectionURL(PROTOCOL + dbPath
			                    + "UserDB/DBSelfIssuedUsers;create=true");
			        }
				 Connection dbConnection = driver.connect(connectionURL, props);
				
				 
				 Statement stmt = dbConnection.createStatement(
							ResultSet.TYPE_SCROLL_INSENSITIVE,
							ResultSet.CONCUR_UPDATABLE);
				 
				 DatabaseMetaData dbmd = dbConnection.getMetaData();
				 ResultSet rs = dbmd.getTables(null, null, "USERS", null);
				 if(rs.next() == false){
					 log.debug("Creating a new table in the database.");
					 stmt = dbConnection.createStatement(
								ResultSet.TYPE_SCROLL_INSENSITIVE,
								ResultSet.CONCUR_UPDATABLE);
					 stmt.executeUpdate("create table USERS(status int, ppid varchar(255), confirm varchar(40))");
					 dbConnection.commit();
				 }else{
					 log.debug("Database and table already found.");
					
				 }
				dbConnection.commit();
				dbConnection.close();
					
						
				
			} catch (Throwable e) {
				log.error(e.getMessage(),e);
				e.printStackTrace();
				
			}
		}
    }

